/*
 *		This Code Was Created By Jeff Molofee 2000
 *		And Modified By Giuseppe D'Agata (waveform@tiscalinet.it)
 *		If You've Found This Code Useful, Please Let Me Know.
 *		Visit My Site At nehe.gamedev.net
 */

#include "glbf.h"

#include <math.h>			// Header File For Windows Math Library
#include <stdio.h>			// Header File For Standard Input/Output
#include <stdlib.h>
#include <string.h>

#if defined(__APPLE__) && defined(__MACH__)
#include <OpenGL/gl.h>	// Header File For The OpenGL32 Library
#include <OpenGL/glu.h>	// Header File For The GLu32 Library
#else
#include <GL/gl.h>	// Header File For The OpenGL32 Library
#include <GL/glu.h>	// Header File For The GLu32 Library
#endif
#include "SDL.h"

static SDL_Surface *LoadBMP(char *filename)
{
	Uint8 *rowhi, *rowlo;
	Uint8 *tmpbuf, tmpch;
	SDL_Surface *image;
	int i, j;

	image = SDL_LoadBMP(filename);
	if ( image == NULL ) {
		fprintf(stderr, "Unable to load %s: %s\n", filename, SDL_GetError());
		return(NULL);
	}

	/* GL surfaces are upsidedown and RGB, not BGR :-) */
	tmpbuf = (Uint8 *)malloc(image->pitch);
	if ( tmpbuf == NULL ) {
		fprintf(stderr, "Out of memory\n");
		return(NULL);
	}
	rowhi = (Uint8 *)image->pixels;
	rowlo = rowhi + (image->h * image->pitch) - image->pitch;
	for ( i=0; i<image->h/2; ++i ) {
		for ( j=0; j<image->w; ++j ) {
			tmpch = rowhi[j*3];
			rowhi[j*3] = rowhi[j*3+2];
			rowhi[j*3+2] = tmpch;
			tmpch = rowlo[j*3];
			rowlo[j*3] = rowlo[j*3+2];
			rowlo[j*3+2] = tmpch;
		}
		memcpy(tmpbuf, rowhi, image->pitch);
		memcpy(rowhi, rowlo, image->pitch);
		memcpy(rowlo, tmpbuf, image->pitch);
		rowhi += image->pitch;
		rowlo -= image->pitch;
	}
	free(tmpbuf);
	return(image);
}

static int LoadGLTextures(int* tex, char* filename, int* width, int* height)									// Load Bitmaps And Convert To Textures
{
    GLuint texture;
    SDL_Surface *TextureImage;			   // Create Storage Space For The Textures
    TextureImage = NULL;

    if (!(TextureImage=LoadBMP(filename)))
    {
        return 1;
    }

    glGenTextures(1, &texture);		  // Create Two Texture

    glBindTexture(GL_TEXTURE_2D, texture);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage->w, TextureImage->h,
                 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->pixels);

    *width = TextureImage->w;
    *height = TextureImage->h;

    SDL_FreeSurface(TextureImage);		// Free The Image Structure

    *tex = texture;

    return 0;								  // Return The Status
}

static int BuildFont(GLBitmapFont* font, char* filename, GLint xsize,
                     GLfloat xdots, GLfloat ydots)								// Build Our Font Display List
{
    int loop;
	float	cx;											// Holds Our X Character Coord
	float	cy;											// Holds Our Y Character Coord
    int xchars, ychars;
    int width, height;
    int ret;

    ret = LoadGLTextures(&font->texture, filename, &width, &height);
    if (ret != 0) {
        SDL_SetError("glbf: cannot load font");
        return 1;
    }

    xchars = (int)(width/xdots);
    ychars = (int)(height/ydots);

	font->base=glGenLists(128);								// Creating 256 Display Lists
	glBindTexture(GL_TEXTURE_2D, font->texture);			// Select Our Font Texture
	for (loop=0; loop<96; loop++)						// Loop Through All 256 Lists
	{
		cx=(float)(loop%xchars)*xdots/width;						// X Position Of Current Character
		cy=(float)(loop/xchars)*ydots/height;						// Y Position Of Current Character

		glNewList(font->base+loop,GL_COMPILE);				// Start Building A List
        glBegin(GL_QUADS);							// Use A Quad For Each Character
        glTexCoord2f(cx,1-cy-ydots/height);			// Texture Coord (Bottom Left)
        glVertex2f(0,0);						// Vertex Coord (Bottom Left)
        glTexCoord2f(cx+xdots/width,1-cy-ydots/height);	// Texture Coord (Bottom Right)
        glVertex2f(xdots,0);						// Vertex Coord (Bottom Right)
        glTexCoord2f(cx+xdots/width,1-cy);			// Texture Coord (Top Right)
        glVertex2f(xdots,ydots);						// Vertex Coord (Top Right)
        glTexCoord2f(cx,1-cy);					// Texture Coord (Top Left)
        glVertex2f(0,ydots);						// Vertex Coord (Top Left)
        glEnd();									// Done Building Our Quad (Character)
        glTranslated(xsize,0,0);						// Move To The Right Of The Character
		glEndList();									// Done Building The Display List
	}													// Loop Until All 256 Are Built
    return 0;
}

static GLvoid KillFont(GLBitmapFont font)									// Delete The Font From Memory
{
    glDeleteTextures(1, &font.texture);
	glDeleteLists(font.base,256);							// Delete All 256 Display Lists
}

static GLvoid glPrint(GLBitmapFont font, char *str)	// Where The Printing Happens
{
	glBindTexture(GL_TEXTURE_2D, font.texture);			// Select Our Font Texture
	glListBase(font.base-32);						// Choose The Font Set (0 or 1)
	glCallLists(strlen(str),GL_BYTE,str);			// Write The Text To The Screen
}

int glbfInit(GLBitmapFont* font, char* filename, GLfloat xsize,
             GLfloat xdots, GLfloat ydots) {
    return BuildFont(font, filename, xsize, xdots, ydots);
}

void glbfQuit(GLBitmapFont font) {
    KillFont(font);
}

void glbfPrint(GLBitmapFont font, char* str) {
    glPrint(font, str);
}
