#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

#include "SDL.h"

#include "SDL_net.h"

#include "SDL_opengl.h"

#include "glbf.h"

#if defined(linux)
#define OS_NAME "Linux"
#elif defined(_WIN32)
#define OS_NAME "Windows"
#elif defined(__APPLE__) && defined(__MACH__)
#define OS_NAME "MacOSX"
#elif defined(__FreeBSD__)
#define OS_NAME "FreeBSD"
#elif defined(sun)
#define OS_NAME "SunOS"
#else
#error please set OS_NAME your self.
#endif

char proxy_host[256];
int proxy_port;
char proxy_user[256];
char proxy_pass[256];
void mutt_to_base64(unsigned char *out, const unsigned char *in,
                    size_t len, size_t olen);

/* Quick utility function for texture creation */
static int power_of_two(int input)
{
	int value = 1;

	while ( value < input ) {
		value <<= 1;
	}
	return value;
}

GLuint SDL_GL_LoadTexture(SDL_Surface *surface, GLfloat *texcoord)
{
	GLuint texture;
	int w, h;
	SDL_Surface *image;
	SDL_Rect area;
	Uint32 saved_flags;
	Uint8  saved_alpha;

	/* Use the surface width and height expanded to powers of 2 */
	w = power_of_two(surface->w);
	h = power_of_two(surface->h);
	texcoord[0] = 0.0f;			/* Min X */
	texcoord[1] = 0.0f;			/* Min Y */
	texcoord[2] = (GLfloat)surface->w / w;	/* Max X */
	texcoord[3] = (GLfloat)surface->h / h;	/* Max Y */

	image = SDL_CreateRGBSurface(
			SDL_SWSURFACE,
			w, h,
			32,
#if SDL_BYTEORDER == SDL_LIL_ENDIAN /* OpenGL RGBA masks */
			0x000000FF, 
			0x0000FF00, 
			0x00FF0000, 
			0xFF000000
#else
			0xFF000000,
			0x00FF0000, 
			0x0000FF00, 
			0x000000FF
#endif
		       );
	if ( image == NULL ) {
		return 0;
	}

	/* Save the alpha blending attributes */
	saved_flags = surface->flags&(SDL_SRCALPHA|SDL_RLEACCELOK);
	saved_alpha = surface->format->alpha;
	if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
		SDL_SetAlpha(surface, 0, 0);
	}

	/* Copy the surface into the GL texture image */
	area.x = 0;
	area.y = 0;
	area.w = surface->w;
	area.h = surface->h;
	SDL_BlitSurface(surface, &area, image, &area);

	/* Restore the alpha blending attributes */
	if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
		SDL_SetAlpha(surface, saved_flags, saved_alpha);
	}

	/* Create an OpenGL texture for the image */
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexImage2D(GL_TEXTURE_2D,
		     0,
		     GL_RGBA,
		     w, h,
		     0,
		     GL_RGBA,
		     GL_UNSIGNED_BYTE,
		     image->pixels);
	SDL_FreeSurface(image); /* No longer needed */

	return texture;
}

/**********************************************************************/

int RunGLTest(int argc, char* argv[], int bpp, float gamma, int full, int mode)
{
	int i;
	int rgb_size[3];
	int w = 640;
	int h = 480;
	int done = 0;
	int frames;
	Uint32 start_time, this_time;
    float color[8][3]= {{ 1.0,  1.0,  0.0}, 
			    { 1.0,  0.0,  0.0},
			    { 0.0,  0.0,  0.0},
			    { 0.0,  1.0,  0.0},
			    { 0.0,  1.0,  1.0},
			    { 1.0,  1.0,  1.0},
			    { 1.0,  0.0,  1.0},
			    { 0.0,  0.0,  1.0}};
	float cube[8][3]= {{ 0.5,  0.5, -0.5}, 
			   { 0.5, -0.5, -0.5},
			   {-0.5, -0.5, -0.5},
			   {-0.5,  0.5, -0.5},
			   {-0.5,  0.5,  0.5},
			   { 0.5,  0.5,  0.5},
			   { 0.5, -0.5,  0.5},
			   {-0.5, -0.5,  0.5}};
	Uint32 video_flags;
	int value;
    static char user[32];
    char* tmpp;
    GLBitmapFont font;
    GLfloat texcoord[4];
    int texture;

	if( SDL_Init( SDL_INIT_VIDEO ) < 0 ) {
		fprintf(stderr,"Couldn't initialize SDL: %s\n",SDL_GetError());
		exit( 1 );
	}

	/* See if we should detect the display depth */
	if ( bpp == 0 ) {
		if ( SDL_GetVideoInfo()->vfmt->BitsPerPixel <= 8 ) {
			bpp = 8;
		} else {
			bpp = 16;  /* More doesn't seem to work */
		}
	}

    video_flags = SDL_OPENGL;

    if (full) {
        video_flags |= SDL_FULLSCREEN;
    }
    else {
        full = 1;
    }

	/* Initialize the display */
	switch (bpp) {
	    case 8:
		rgb_size[0] = 3;
		rgb_size[1] = 3;
		rgb_size[2] = 2;
		break;
	    case 15:
	    case 16:
		rgb_size[0] = 5;
		rgb_size[1] = 5;
		rgb_size[2] = 5;
		break;
            default:
		rgb_size[0] = 8;
		rgb_size[1] = 8;
		rgb_size[2] = 8;
		break;
	}
	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, rgb_size[0] );
	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, rgb_size[1] );
	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, rgb_size[2] );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

	if ( SDL_SetVideoMode( w, h, bpp, video_flags ) == NULL ) {
		fprintf(stderr, "Couldn't set GL mode: %s\n", SDL_GetError());
		SDL_Quit();
		exit(1);
	}

	printf("Screen BPP: %d\n", SDL_GetVideoSurface()->format->BitsPerPixel);
	printf("\n");
	printf( "Vendor     : %s\n", glGetString( GL_VENDOR ) );
	printf( "Renderer   : %s\n", glGetString( GL_RENDERER ) );
	printf( "Version    : %s\n", glGetString( GL_VERSION ) );
	printf( "Extensions : %s\n", glGetString( GL_EXTENSIONS ) );
	printf("\n");

	SDL_GL_GetAttribute( SDL_GL_RED_SIZE, &value );
	printf( "SDL_GL_RED_SIZE: requested %d, got %d\n", rgb_size[0],value);
	SDL_GL_GetAttribute( SDL_GL_GREEN_SIZE, &value );
	printf( "SDL_GL_GREEN_SIZE: requested %d, got %d\n", rgb_size[1],value);
	SDL_GL_GetAttribute( SDL_GL_BLUE_SIZE, &value );
	printf( "SDL_GL_BLUE_SIZE: requested %d, got %d\n", rgb_size[2],value);
	SDL_GL_GetAttribute( SDL_GL_DEPTH_SIZE, &value );
	printf( "SDL_GL_DEPTH_SIZE: requested %d, got %d\n", bpp, value );
	SDL_GL_GetAttribute( SDL_GL_DOUBLEBUFFER, &value );
	printf( "SDL_GL_DOUBLEBUFFER: requested 1, got %d\n", value );

	/* Set the window manager title bar */
	SDL_WM_SetCaption( "SDL GL test", "testgl" );

	/* Set the gamma for the window */
	if ( gamma != 0.0 ) {
		SDL_SetGamma(gamma, gamma, gamma);
	}

	glViewport( 0, 0, w, h );
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );

	glOrtho( -2.0, 2.0, -2.0, 2.0, -20.0, 20.0 );

	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity( );

	glEnable(GL_DEPTH_TEST);

	glDepthFunc(GL_LESS);

	glShadeModel(GL_SMOOTH);

    glbfInit(&font, "kochi.bmp", 12, 14, 21);

    if (user[0] == '\0') {
        int done = 0;
        int i = 0;

        tmpp = getenv("USER");
        if (tmpp) {
            strncpy(user, tmpp, 31);
            i = strlen(user);
        }
        else {
            user[0] = '\0';
        }

        while (!done) {
            SDL_Event ev;

            glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

            glColor3f(1,1,1);

            glEnable(GL_TEXTURE_2D);

            glPushMatrix();
            glTranslatef(-1.4, 0, 0);
            glScalef(0.007, 0.007, 1);
            glbfPrint(font, "please input your name with keyboard");
            glPopMatrix();

            glPushMatrix();
            glTranslatef(-1.3, -0.6, 0);
            glScalef(0.007, 0.007, 1);
            glbfPrint(font, "push return key to go to benchmark");
            glPopMatrix();

            glPushMatrix();
            glTranslatef(-1.3, -1.2, 0);
            glScalef(0.007, 0.007, 1);
            glbfPrint(font, "push backspace key to modify input");
            glPopMatrix();

            glPushMatrix();
            glTranslatef(-1.5, 0.6, 0);
            glScalef(0.007, 0.007, 1);
            glbfPrint(font, user);
            glPopMatrix();

            SDL_GL_SwapBuffers();

            while( SDL_PollEvent( &ev) ) {
                if (ev.type != SDL_KEYDOWN) continue;

                switch (ev.key.keysym.sym) {
                case SDLK_BACKSPACE:
                    if (i > 0) {
                        i--;
                        user[i] = '\0';
                    }
                    break;

                case SDLK_RETURN:
                    done = 1;
                    break;

                case SDLK_ESCAPE:
                    printf("keyboard exit\n");
                    exit(0);

                default:
                    if (isprint(ev.key.keysym.sym) && i < 31) {
                        user[i] = ev.key.keysym.sym;
                        i++;
                        user[i] = '\0';
                    }
                }
            }

            SDL_Delay(10);
        }
    }

    if (mode == 1) {
        SDL_Surface* tmp = SDL_LoadBMP("icon.bmp");
        if (tmp == NULL) {
            printf("icon.bmp not found\n");
            exit(1);
        }
        texture = SDL_GL_LoadTexture(tmp, texcoord);
        SDL_FreeSurface(tmp);

        glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, texture);
    }
    else if (mode == 2) {
        glDisable(GL_TEXTURE_2D);
        glEnable(GL_LIGHTING);
        glEnable(GL_LIGHT0);
        glEnable(GL_COLOR_MATERIAL);
    }
    else {
        glDisable(GL_TEXTURE_2D);
    }

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/* Loop until done. */
	start_time = SDL_GetTicks();
	frames = 0;
	while( start_time + 10000 > SDL_GetTicks() ) {
		GLenum gl_error;
		char* sdl_error;
		SDL_Event event;

		/* Do our drawing, too. */
		glClearColor( 0.0, 0.0, 0.0, 1.0 );
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		glBegin( GL_QUADS );

        switch (mode) {
        case 0:
			glColor3fv(color[0]);
			glVertex3fv(cube[0]);
			glColor3fv(color[1]);
			glVertex3fv(cube[1]);
			glColor3fv(color[2]);
			glVertex3fv(cube[2]);
			glColor3fv(color[3]);
			glVertex3fv(cube[3]);
			
			glColor3fv(color[3]);
			glVertex3fv(cube[3]);
			glColor3fv(color[4]);
			glVertex3fv(cube[4]);
			glColor3fv(color[7]);
			glVertex3fv(cube[7]);
			glColor3fv(color[2]);
			glVertex3fv(cube[2]);
			
			glColor3fv(color[0]);
			glVertex3fv(cube[0]);
			glColor3fv(color[5]);
			glVertex3fv(cube[5]);
			glColor3fv(color[6]);
			glVertex3fv(cube[6]);
			glColor3fv(color[1]);
			glVertex3fv(cube[1]);
			
			glColor3fv(color[5]);
			glVertex3fv(cube[5]);
			glColor3fv(color[4]);
			glVertex3fv(cube[4]);
			glColor3fv(color[7]);
			glVertex3fv(cube[7]);
			glColor3fv(color[6]);
			glVertex3fv(cube[6]);

			glColor3fv(color[5]);
			glVertex3fv(cube[5]);
			glColor3fv(color[0]);
			glVertex3fv(cube[0]);
			glColor3fv(color[3]);
			glVertex3fv(cube[3]);
			glColor3fv(color[4]);
			glVertex3fv(cube[4]);

			glColor3fv(color[6]);
			glVertex3fv(cube[6]);
			glColor3fv(color[1]);
			glVertex3fv(cube[1]);
			glColor3fv(color[2]);
			glVertex3fv(cube[2]);
			glColor3fv(color[7]);
			glVertex3fv(cube[7]);

            break;

        case 1:
			glColor3fv(color[0]);
            glTexCoord2f(texcoord[0], texcoord[1]);
			glVertex3fv(cube[0]);
			glColor3fv(color[1]);
            glTexCoord2f(texcoord[2], texcoord[1]);
			glVertex3fv(cube[1]);
			glColor3fv(color[2]);
            glTexCoord2f(texcoord[2], texcoord[3]);
			glVertex3fv(cube[2]);
			glColor3fv(color[3]);
            glTexCoord2f(texcoord[0], texcoord[3]);
			glVertex3fv(cube[3]);
			
			glColor3fv(color[3]);
            glTexCoord2f(texcoord[0], texcoord[1]);
			glVertex3fv(cube[3]);
			glColor3fv(color[4]);
            glTexCoord2f(texcoord[2], texcoord[1]);
			glVertex3fv(cube[4]);
			glColor3fv(color[7]);
            glTexCoord2f(texcoord[2], texcoord[3]);
			glVertex3fv(cube[7]);
			glColor3fv(color[2]);
            glTexCoord2f(texcoord[0], texcoord[3]);
			glVertex3fv(cube[2]);
			
			glColor3fv(color[0]);
            glTexCoord2f(texcoord[0], texcoord[1]);
			glVertex3fv(cube[0]);
			glColor3fv(color[5]);
            glTexCoord2f(texcoord[2], texcoord[1]);
			glVertex3fv(cube[5]);
			glColor3fv(color[6]);
            glTexCoord2f(texcoord[2], texcoord[3]);
			glVertex3fv(cube[6]);
			glColor3fv(color[1]);
            glTexCoord2f(texcoord[0], texcoord[3]);
			glVertex3fv(cube[1]);
			
			glColor3fv(color[5]);
            glTexCoord2f(texcoord[0], texcoord[1]);
			glVertex3fv(cube[5]);
			glColor3fv(color[4]);
            glTexCoord2f(texcoord[2], texcoord[1]);
			glVertex3fv(cube[4]);
			glColor3fv(color[7]);
            glTexCoord2f(texcoord[2], texcoord[3]);
			glVertex3fv(cube[7]);
			glColor3fv(color[6]);
            glTexCoord2f(texcoord[0], texcoord[3]);
			glVertex3fv(cube[6]);

			glColor3fv(color[5]);
            glTexCoord2f(texcoord[0], texcoord[1]);
			glVertex3fv(cube[5]);
			glColor3fv(color[0]);
            glTexCoord2f(texcoord[2], texcoord[1]);
			glVertex3fv(cube[0]);
			glColor3fv(color[3]);
            glTexCoord2f(texcoord[2], texcoord[3]);
			glVertex3fv(cube[3]);
			glColor3fv(color[4]);
            glTexCoord2f(texcoord[0], texcoord[3]);
			glVertex3fv(cube[4]);

			glColor3fv(color[6]);
            glTexCoord2f(texcoord[0], texcoord[1]);
			glVertex3fv(cube[6]);
			glColor3fv(color[1]);
            glTexCoord2f(texcoord[2], texcoord[1]);
			glVertex3fv(cube[1]);
			glColor3fv(color[2]);
            glTexCoord2f(texcoord[2], texcoord[3]);
			glVertex3fv(cube[2]);
			glColor3fv(color[7]);
            glTexCoord2f(texcoord[0], texcoord[3]);
			glVertex3fv(cube[7]);

            break;

        case 2:
            glNormal3f(0, 0, -1);
			glColor3fv(color[0]);
			glVertex3fv(cube[0]);
			glColor3fv(color[1]);
			glVertex3fv(cube[1]);
			glColor3fv(color[2]);
			glVertex3fv(cube[2]);
			glColor3fv(color[3]);
			glVertex3fv(cube[3]);

            glNormal3f(-1, 0, 0);
			glColor3fv(color[3]);
			glVertex3fv(cube[3]);
			glColor3fv(color[4]);
			glVertex3fv(cube[4]);
			glColor3fv(color[7]);
			glVertex3fv(cube[7]);
			glColor3fv(color[2]);
			glVertex3fv(cube[2]);

			glNormal3f(1, 0, 0);
			glColor3fv(color[0]);
			glVertex3fv(cube[0]);
			glColor3fv(color[5]);
			glVertex3fv(cube[5]);
			glColor3fv(color[6]);
			glVertex3fv(cube[6]);
			glColor3fv(color[1]);
			glVertex3fv(cube[1]);

			glNormal3f(0, 0, 1);
			glColor3fv(color[5]);
			glVertex3fv(cube[5]);
			glColor3fv(color[4]);
			glVertex3fv(cube[4]);
			glColor3fv(color[7]);
			glVertex3fv(cube[7]);
			glColor3fv(color[6]);
			glVertex3fv(cube[6]);

            glNormal3f(0, 1, 0);
			glColor3fv(color[5]);
			glVertex3fv(cube[5]);
			glColor3fv(color[0]);
			glVertex3fv(cube[0]);
			glColor3fv(color[3]);
			glVertex3fv(cube[3]);
			glColor3fv(color[4]);
			glVertex3fv(cube[4]);

            glNormal3f(0, -1, 0);
			glColor3fv(color[6]);
			glVertex3fv(cube[6]);
			glColor3fv(color[1]);
			glVertex3fv(cube[1]);
			glColor3fv(color[2]);
			glVertex3fv(cube[2]);
			glColor3fv(color[7]);
			glVertex3fv(cube[7]);

            break;

        default:
            printf("something strange\n");
            exit(1);
        }

		glEnd( );
		
		glMatrixMode(GL_MODELVIEW);
		glRotatef(5.0, 1.0, 1.0, 1.0);

		SDL_GL_SwapBuffers( );

		/* Check for error conditions. */
		gl_error = glGetError( );

		if( gl_error != GL_NO_ERROR ) {
			fprintf( stderr, "testgl: OpenGL error: %d\n", gl_error );
		}

		sdl_error = SDL_GetError( );

		if( sdl_error[0] != '\0' ) {
			fprintf(stderr, "testgl: SDL error '%s'\n", sdl_error);
			SDL_ClearError();
		}

		/* Check if there's a pending event. */
/*
		while( SDL_PollEvent( &event ) ) {
			done = HandleEvent(&event);
		}
*/
		++frames;
	}

	/* Print out the frames per second */
	this_time = SDL_GetTicks();
	if ( this_time != start_time ) {
		printf("%2.2f FPS\n",
               ((float)frames/(this_time-start_time))*1000.0);
	}

    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);
    glDisable(GL_LIGHT0);
    glDisable(GL_COLOR_MATERIAL);

    {
        glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        glColor3f(1,1,1);

        glEnable(GL_TEXTURE_2D);

        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();

        glPushMatrix();
        glTranslatef(-1.4, 0, 0);
        glScalef(0.007, 0.007, 1);
        glbfPrint(font, "reporting to score server...");
        glPopMatrix();

        SDL_GL_SwapBuffers();    
    }

    {
        IPaddress serverIP;
        TCPsocket tcpsock;
        SDL_Surface* screen;
        char buf[40960];
        char bod[40960];
        char b[256];
        int len;

        bod[0] = '\0';
        strcat(bod, "Vendor: ");
        strcat(bod, glGetString(GL_VENDOR));
        strcat(bod, "\n");
        strcat(bod, "Renderer: ");
        strcat(bod, glGetString(GL_RENDERER));
        strcat(bod, "\n");
        strcat(bod, "Version: ");
        strcat(bod, glGetString(GL_VERSION));
        strcat(bod, "\n");
        strcat(bod, "Extensions: ");
        strcat(bod, glGetString(GL_EXTENSIONS));
        strcat(bod, "\n");
        strcat(bod, "GLU-Version: ");
        strcat(bod, gluGetString(GLU_VERSION));
        strcat(bod, "\n");
        strcat(bod, "GLU-Extensions: ");
        strcat(bod, gluGetString(GLU_EXTENSIONS));
        strcat(bod, "\n");
        if (mode == 1) {
            strcat(bod, "Option: texture\n");
        }
        else if (mode == 2) {
            strcat(bod, "Option: lighting\n");
        }
        strcat(bod, "User: ");
        strcat(bod, user);
        strcat(bod, "\n");
        strcat(bod, "OS: ");
        strcat(bod, OS_NAME);
        strcat(bod, "\n");
        strcat(bod, "FPS: ");
        sprintf(b, "%d", (int)(((float)frames/(this_time-start_time))*1000.0));
        strcat(bod, b);
        strcat(bod, "\n");
        strcat(bod, "bpp: ");
        sprintf(b, "%d", SDL_GetVideoSurface()->format->BitsPerPixel);
        strcat(bod, b);
        strcat(bod, "\n");
        strcat(bod, "full: ");
        screen = SDL_GetVideoSurface();
        strcat(bod, (screen->flags&SDL_FULLSCREEN) ? "yes" : "no");
        strcat(bod, "\n");

        buf[0] = '\0';
        strcat(buf, "POST http://shinh.skr.jp/sdlbench/registtestgl.cgi HTTP/1.0\r\n");
        if (proxy_user[0]) {
            char bp[512];
            strcat(buf, "Proxy-Authorization: Basic ");
            sprintf(bp, "%s:%s", proxy_user, proxy_pass);
            mutt_to_base64(buf+strlen(buf), bp, strlen(bp), 2048);
            strcat(buf, "\r\n");
        }
        strcat(buf, "Host: ");
        strcat(buf, proxy_host);
        strcat(buf, ":");
        sprintf(b, "%d", proxy_port);
        strcat(buf, b);
        strcat(buf, "\r\n");
        strcat(buf, "Content-Length: ");
        sprintf(b, "%d", strlen(bod));
        strcat(buf, b);
        strcat(buf, "\r\n");
        strcat(buf, "\r\n");
        strcat(buf, bod);

        if ( SDLNet_Init() < 0 ) {
            fprintf(stderr, "Couldn't initialize net: %s\n",
                    SDLNet_GetError());
            exit(1);
        }

        SDLNet_ResolveHost(&serverIP, proxy_host, proxy_port);
        if (serverIP.host == INADDR_NONE) {
            fprintf(stderr, "Couldn't resolve hostname: %s\n",
                    SDLNet_GetError());
            exit(1);
        }

        tcpsock = SDLNet_TCP_Open(&serverIP);
        if (tcpsock == NULL) {
            fprintf(stderr, "Connect failed %s\n",
                    SDLNet_GetError());
            exit(1);
        }

        /* printf(buf); */
        SDLNet_TCP_Send(tcpsock, buf, strlen(buf));

        len = SDLNet_TCP_Recv(tcpsock, buf, 40960);
        buf[len] = '\0';
        printf(buf);

        SDLNet_TCP_Close(tcpsock);

        SDLNet_Quit();
    }

    glbfQuit(font);
    glDeleteTextures(1, &texture);

	/* Destroy our GL context, etc. */
	SDL_Quit();
	return(0);
}

static void load_proxy_cfg() {
    FILE* fp = fopen("proxy.cfg", "r");
    char buf[256];

    if (!fp) {
        printf("proxy.cfg: not found. we do not use proxy...");
        return;
    }

    while (!feof(fp)) {
        fgets(buf, 256, fp);
        if (strlen(buf) > 0 && buf[strlen(buf)-1] == '\n') {
            buf[strlen(buf)-1] = '\0';
        }
        if (strlen(buf) > 0 && buf[strlen(buf)-1] == '\r') {
            buf[strlen(buf)-1] = '\0';
        }

        if (strstr(buf, "proxyHost: ") == buf) {
            strcpy(proxy_host, buf+11);
        }
        else if (strstr(buf, "proxyPort: ") == buf) {
            proxy_port = atoi(buf+11);
        }
        else if (strstr(buf, "proxyUser: ") == buf) {
            strcpy(proxy_user, buf+11);
        }
        else if (strstr(buf, "proxyPass: ") == buf) {
            strcpy(proxy_pass, buf+11);
        }
    }
}

int main(int argc, char *argv[])
{
	int i;
	int bpp = 0;
	float gamma = 0.0;
    int full = 0;

    strcpy(proxy_host, "shinh.skr.jp");
    proxy_port = 80;
    proxy_user[0] = '\0';
    proxy_pass[0] = '\0';
    load_proxy_cfg();

	for ( i=1; argv[i]; ++i ) {
		if ( strcmp(argv[i], "-bpp") == 0 ) {
 		       bpp = atoi(argv[++i]);
		}
		if ( strcmp(argv[i], "-gamma") == 0 ) {
 		       gamma = (float)atof(argv[++i]);
		}
		if ( strcmp(argv[i], "-fullscreen") == 0 ) {
			full = 1;
		}
		if ( strncmp(argv[i], "-h", 2) == 0 ) {
            printf("Usage: %s [-bpp n] [-gamma n]\n",
 			      argv[0]);
			exit(0);
		}
	}

    RunGLTest(argc, argv, bpp, gamma, full, 0);
    RunGLTest(argc, argv, bpp, gamma, full, 1);
    RunGLTest(argc, argv, bpp, gamma, full, 2);
    RunGLTest(argc, argv, bpp, gamma, !full, 0);
    RunGLTest(argc, argv, bpp, gamma, !full, 1);
    RunGLTest(argc, argv, bpp, gamma, !full, 2);

	return 0;
}

